# ----------------------------------------------------------------------------
# Csound Tcl-Music-Script                                          OCT-07-2002
# Copyright (c) 2002 by Hartmut Schwarz               matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# Grundwissen der Musik fuer den Tcl-Interpreter, Lektion x
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
puts "Lektion x : Ein einfaches Instrument, a simple instrument";
# ----------------------------------------------------------------------------

proc instr_begin_notes rCTX {
 global $rCTX;
 store_i_statement $rCTX; # loeschen
}

proc instr_key_to_pitch { rCTX nkey } {
# wandelt Tastennummer in Csound-Tonhoehe (pitch)
 global $rCTX;
 if { [string is double $nkey] } {
  set key [expr $nkey + [TREEget $rCTX kybs]];
  set oct [expr 7.0 + $key / 12.0 ];
  set ptc [format "%9.3f" [expr int($oct) + ($oct - int($oct))*0.12]];
 } else {
 set ptc $nkey; # z.B. Pause
 }
 return $ptc;
}
   
proc instr_play_note { rCTX iocn inum crtv nval when\
                       dura volm loud nkey meta csym } {
# rCTX = reference to music context
# iocn = symbol index (i.e. octave counter in case of play_chord, otherwise 0) 
# inum = instrument number
# crtv = current time value
# nval = time value duration
# when = current real time
# dura = real time duration
# volm = attack power
# loud = volume modifier
# nkey = key number
# meta = meta info (empty in case of play_chord)
# csym = chord symbol (empty in case of play_chord)
 global $rCTX, STAT_NumberOfHits;
 set out  [TREEget $rCTX out];
 set idst [TREEget $rCTX idst];
 set iazm [TREEget $rCTX iazm];
 set isze [TREEget $rCTX isze];
 set iang [TREEget $rCTX iang];
 set ptch [instr_key_to_pitch $rCTX $nkey];
 if { ![string is double $ptch] || $dura == 0 } {
  # --- Pause nur als Kommentar ausgeben ---
  puts $out [format "; *** Pause %8.3f %8.3f ***" $when $dura];
 } else {
  # --- Haltedauer pro Notenwert beruecksichtigen ---
  set dura [expr $dura*[TREEget $rCTX tvtv]];
  # --- gueltige Note speichern oder spielen ---
  set p1  [format "i %2d" $inum];
  set p2  [format "%8.3f" $when];
  set p3  [format "%8.3f" $dura];
  set p4  [format "%8.3f" $volm];
  set p5  [format "%16s"  $ptch];
  set p6  [format "%8.3f" $loud];
  set p7  [format "%4d" [expr round($idst)]];
  set p8  [format "%4d" [expr round($iazm)]];
  set p9  [format "%4d" [expr round($isze)]];
  set p10 [format "%4d" [expr round($iang)]];
  set p11 [format " ; %6d %8.3f %8.3f" $STAT_NumberOfHits $crtv $nval];

  # --- Bei Notenwechsel gespeicherte Note spielen ---
  if {      [array names ${rCTX} "iPar5"] != ""\
      && ( ![TREEget $rCTX lega] || [set ${rCTX}(iPar5)] != $p5 ) } {
   flush_i_statement $rCTX;
  }
  # --- Bei Notengleichheit und Legato, gespeicherte Note fortschreiben ---
  if {   [array names ${rCTX} "iPar5"] != ""\
      && [TREEget $rCTX lega] && [set ${rCTX}(iPar5)] == $p5 } {
   set ${rCTX}(iPar3) [expr [set ${rCTX}(iPar3)] + $p3];
  # --- Bei Legato ohne Notengleichheit, Note speichern ---
  } elseif { [TREEget $rCTX lega] } {
   store_i_statement $rCTX $p1 $p2 $p3 $p4 $p5 $p6 $p7 $p8 $p9 $p10 $p11;
  # --- alle anderen Faelle direkt spielen ---
  } else {
   store_i_statement $rCTX $p1 $p2 $p3 $p4 $p5 $p6 $p7 $p8 $p9 $p10 $p11;
   flush_i_statement $rCTX;
  }
 }
}
   
proc instr_beat_note { rCTX inum crtv nval when\
                       dura volm loud nkey meta csym } {
 global $rCTX;
 set bell [TREEget $rCTX bell];
 if { $bell != "" } {
  global $bell;
 }
 if { [array exists $bell] } {
  set beats [beat_advance $rCTX $crtv $nval $when $dura]; 
  foreach {tm bt br ht} $beats {
   array set $bell "crtm $tm";
   touch $bell 1;
  }
 } 
}

proc instr_end_notes rCTX {
 global $rCTX;
 flush_i_statement $rCTX;
}
