; --------------------------------------------------------------------------------
; Copyright (c) 2003-2004 by Hartmut Schwarz, matut@matut-spielwerk.de
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2 
; of the License, or (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public
; License along with this program; if not, write to the Free
; Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
; MA 02111-1307, USA.
; --------------------------------------------------------------------------------

sr     = 44100
kr     = 11025
ksmps  =     4
nchnls =     2

; --------------------------------------------------------------------------------
         instr 1 ; Ethereal Organ
; --------------------------------------------------------------------------------

         ; feature switches ( 1 = ON, 0 = OFF )
idstamp  = 1                               ; Abstandsamplitude
idirpha  = 0                               ; Richtungsphase
idiramp  = 1                               ; Richtungsamplitude
idirflt  = 0                               ; Richtungsfilter
ivibrato = 1                               ; Vibrato

; --------------------------------------------------------------------------------
;                        P A R A M E T E R S
; --------------------------------------------------------------------------------

         ; original parameters (p1 = instrument number, p2 = initial time)
idur     = p3                              ; Anschlagdauer
ivol     = p4                              ; Anschlagstaerke
icps     = p5                              ; Tonfrequenz
imix     = p6                              ; Mixfaktor
idst     = p7                              ; Hoerabstand
iazm     = p8                              ; Richtungswinkel (Azimut)
isze     = p9                              ; Groessenfaktor
iang     = p10                             ; Aufstellwinkel des Instrumentes

         ; calculated parameters
itime    times                                ; absolute performance time in seconds
ipha     = frac(icps*itime)                   ; phase
iamp     = ampdb(ivol)                        ; Amplitude
ioct     = octcps(icps)                       ; Oktave
ipch     = pchoct(ioct)                       ; Tonhoehe
isazm    = sin(3.141592654*iazm/180)          ; Sinus des Azimut
icazm    = cos(3.141592654*iazm/180)          ; Cosinus des Azimut
isang    = sin(3.141592654*((iazm+iang)/180)) ; Sinus des Aufstellwinkels
icang    = cos(3.141592654*((iazm+iang)/180)) ; Cosinus des Aufstellwinkels
         ; (der Aufstellwinkel gilt relativ zum Azimut)

; --------------------------------------------------------------------------------
;              A L L G E M E I N E   S I G N A L G R O E S S E N
; --------------------------------------------------------------------------------

         ; f-Tab Nummernliste fuer Tcl-Auswertung
ift1     = 1
ift2     = 2
ift3     = 3
ift4     = 4
ift5     = 5
ift6     = 6
ift8     = 8

         ; sonstige Konstanten
ihead    =    124                           ; Abstand von linkem zum rechtem Ohr
ivsnd    = 331300                           ; Schallgeschwindigkeit [mm/s]
iend     =  0.001                           ; relative Endamplitude
id0      =  2/sr                            ; Basisverzoegerung (delay 0 geht nicht)

         ; aktuelle Instrumentenzeit (Sekunden) bestimmen
ktime    timeinsts

         ; Lokalisierungsgroessen
ixscale  = frac(ipch)/.11-.5                 ; relative Tonleiterposition
ixoctav  = frac(pchoct(octcps(icps)))/.11-.5 ; relative Oktavposition

         ; Richtcharakteristik des Ohres
iiface   = 0.5                            ; Amplitude mit dem Gesicht zum Schall
iihere   = 3.0                            ; Amplitude dem Schall zugewandt
iiback   = 0.5                            ; Amplitude mit dem Ruecken zum Schall
iiaway   = 0.2                            ; Amplitude vom Schall abgewandt
ihhere   = 6600^2;                        ; Grenzfrequenz dem Schall zugewandt
ihface   = 4400^2;                        ; Grenzfrequenz mit dem Gesicht zum Schall
ihback   = 2200^2;                        ; Grenzfrequenz mit dem Ruecken zum Schall
ihaway   = 1100^2;                        ; Grenzfrequenz vom Schall abgewandt    

; --------------------------------------------------------------------------------
;                    R E G I S T E R V E R T E I L U N G
; --------------------------------------------------------------------------------
;
;                         It could feel so bad.
;                         But it could feel so good.
;                         It could sing you to sleep
;                         [Kate Bush, Experiment IV]
;
; --------------------------------------------------------------------------------

;        reiner und unreiner Anteil anschlagabhaengig
imingood =   0.01  ; minimaler reiner Anteil
imaxgood =  99.99  ; maximaler reiner Anteil
ivgood   =  72.5   ; Anschlagstaerke fuer reinsten Klang (ibad = iyminbad)
ivdist   =   5.0   ; Abstand ivgood zum unreinsten Klang (ibad = iymaxbad)
iv1      = ivgood - ivdist;
iv2      = ivgood - ivdist/2;
iv3      = ivgood;
iv4      = ivgood + ivdist/2;
iv5      = ivgood + ivdist;
igood    = (ivol>iv1?2*((ivol-iv1)/ivdist)^2:0);
igood    = (ivol>iv2?1-2*((iv3-ivol)/ivdist)^2:igood);
igood    = (ivol>iv3?1-2*((ivol-iv3)/ivdist)^2:igood);
igood    = (ivol>iv4?2*((iv5-ivol)/ivdist)^2:igood);
igood    = (ivol>iv5?0:igood);
igood    = igood*(imaxgood-imingood)+imingood
ibad     = 100-igood
;        Verteilung auf die unterschiedlichen Register
i0i      = 0                                 ; Anteil Register 0 (Sinusoszillator)
i1i      = igood                             ; Anteil Register 1 (Sinusoszillator)
i2i      = 0                                 ; Anteil Register 2 (FM Oszillator)
i3i      = ibad                              ; Anteil Register 3 (Buzzer)

; --------------------------------------------------------------------------------
;                             R E G I S T E R   0
; --------------------------------------------------------------------------------
         reg0:
i0total  = 0
a0m      = 0
a0l      = 0
a0r      = 0
         if i0i == 0 goto reg1

;--------------------------------------------------------------- i-rate action ---
	 ; Konstanten
i0ft     = ift1       ; f-Tab Nummer f. Sinusoszillator
i0ftvib  = ift5       ; f-Tab f. Vibrato
iNwid    = 1200       ; Baubreite [mm]
iNdst    = 1000       ; Hoerabstand [mm]

         ; spacial phase and amplitude modifications
iNx0     = ixoctav*(isze*iNwid/100);         ; x,y-Position der Schallquelle
iNy0     = 0;                                ; innerhalb des Instrumentes
iNx      = iNx0*icang-(idst*iNdst/100.0)*isazm-iNy0*isang ; x,y-Position der Schallquelle
iNy      = (idst*iNdst/100.0)*icazm+iNx0*isang+iNy0*icang ; relativ zum Zuhoerer
iNxl     = iNx+ihead/2                       ; x-Position relativ zum linken Ohr
iNxr     = iNx-ihead/2                       ; x-Position relativ zum rechten Ohr 
iNdstl2  = iNy^2+iNxl^2                      ; square of left ear distance
iNdstr2  = iNy^2+iNxr^2                      ; square of right ear distance
iNdstl   = sqrt(iNdstl2)                     ; left ear distance [mm]
iNdstr   = sqrt(iNdstr2)                     ; right ear distance [mm]
iNpha    = (iNdstr-iNdstl)/ivsnd             ; Phasendifferenz rechts - links [s]
iNilx    = (iNxl<=0?iihere:iiaway)           ; x-Richtungskoeffizient linkes Ohr
iNirx    = (iNxr>=0?iihere:iiaway)           ; x-Richtungskoeffizient rechtes Ohr
iNiy     = (iNy>0?iiface:iiback)             ; y-Richtungskoeffizient links u. rechts
iNhlx    = (iNxl<=0?ihhere:ihaway)           ; x-Grenzfrequenz linkes Ohr
iNhrx    = (iNxr>=0?ihhere:ihaway)           ; x-Grenzfrequenz rechtes Ohr
iNhy     = (iNy>0?ihface:ihback)             ; y-Grenzfrequenz links u. rechts
         ; Gesamt-Richtungskoeffizienten und -Grenzfrequenzen links u. rechts
iNil     = (iNdstl==0?iNilx:sqrt((iNilx*iNxl^2+iNiy*iNy^2)/iNdstl^2)); 
iNir     = (iNdstr==0?iNirx:sqrt((iNirx*iNxr^2+iNiy*iNy^2)/iNdstr^2)); 
i0hl     = (iNdstl==0?iNhlx:sqrt((iNhlx*iNxl^2+iNhy*iNy^2)/iNdstl^2)); 
i0hr     = (iNdstr==0?iNhrx:sqrt((iNhrx*iNxr^2+iNhy*iNy^2)/iNdstr^2)); 
         ; amplitude modification by distance
iNdl     = (iNdstl>iNdst?iNdst/iNdstl:.5*(3-(iNdstl/iNdst)^2))
iNdr     = (iNdstr>iNdst?iNdst/iNdstr:.5*(3-(iNdstr/iNdst)^2))

         ; Daempfungskoeffizienten abhaengig von Tonhoehe
         ; Anschlagdauer, Anschlagstaerke
iNa      = 1+8*(icps/4000)^2               ; frequenzabhaengiger Ausgangswert
iNa      = iNa*(1+.2*exp(ivol/76.0-1))     ; abhaengig von Anschlagstaerke
iNa      = iNa*(.6+.6*exp(-idur))          ; abhaengig von Anschlagsdauer
iNb      = iNa                             ; Freischwingdaempfung
iNc      = 20*iNa                          ; Daempferdaempfung

         ; Anstiegs-/Freischwing-/Daempfungs-Amplituden und -Zeiten 
         ; abhaengig von Tonhoehe, Anschlagstaerke, Anschlagsdauer
i0start  = 1+((ioct-9)^2)/48               ; relative Anfangsamplitude
i0rise   = (12-ivol/19)/icps               ; Anstiegszeit
i0rise   = (i0rise<.001?.001:i0rise)       ; Minimum einhalten
i0rise   = (i0rise>(idur/2)?idur/2:i0rise) ; Maximum einhalten
iNfree   = idur - i0rise                   ; Freischwingzeit
i0rest   = i0start*exp(-iNb*iNfree)        ; relative Freischwing-Restamplitude
i0rest   = (i0rest==0?0.000001:i0rest)     ; darf nicht 0 sein wegen envlpx
i0dec    = -(log(iend)+iNb*iNfree)/iNc     ; Daempfungszeit
i0dec    = (i0dec < 0 ? .001 : i0dec)      ; wenn iend schon in Freischwing-Phase
i0total  = i0rise + iNfree + i0dec         ; Spieldauer

         ; Vibratoamplitude und Frequenz abhaengig von 
         ; Tonhoehe und Anschlagstaerke
i0vamp   = (ivibrato!=0?(ivol<ivgood?(idur<1?0:1):0):0)   ; Amplitude
i0vcps   = 4.5+0.1*ioct                                   ; Frequenz
i0vtime  = .333*idur                                      ; Zeitpunkt

         ; Frequenzabweichung abhaengig von Anschlag
i0dev    = 0.0006*((ivol/90)^2) ; relative Abweichung fuer Frequenzverlauf

         ; spacial signal modifications
i0left   = (idiramp!=0?.5*iNil:.5)            ; left amplitude modified by direction
i0right  = (idiramp!=0?.5*iNir:.5)            ; right amplitude modified by direction
i0left   = (idstamp!=0?iNdl*i0left:i0left)    ; left amplitude modified by distance
i0right  = (idstamp!=0?iNdr*i0right:i0right)  ; right amplitude modified by distance
i0dell   = (idirpha!=0?(iNpha>0?id0:id0-iNpha):id0) ; left phase modified by direction
i0delr   = (idirpha!=0?(iNpha>0?id0+iNpha:id0):id0) ; right phase modified by direction

;--------------------------------------------------------------- k-rate action ---
         ; Einhuellende
kenv     envlpx i0start, i0rise, i0total, i0dec, ift8, i0rest/i0start, iend/i0rest
kenv     = (ktime>i0total?0:kenv)

         ; Vibratoamplitude
kvosc    poscil i0vamp, i0vcps, i0ftvib, 0.25
kvenv    linseg 0, i0vtime, 0, i0total-i0vtime, 1
kvenv    = (ktime>i0total?0:kvenv)
kvib     = 1+kvenv*kvosc

         ; resultierender Amplitudenverlauf
k0amp    = imix*iamp*kvib*kenv

         ; Frequenzverlauf
k0fenv   linseg 1+i0dev, i0total, 1

;--------------------------------------------------------------- a-rate action ---
         ; Signalerzeugung
a0m      poscil k0amp, k0fenv*icps, i0ft, ipha
a0dl     delay i0left*a0m, i0dell
a0dr     delay i0right*a0m, i0delr
	 if idirflt == 0 goto noflt0
a0l      tone a0dl, i0hl
a0r      tone a0dr, i0hr
         goto reg1
         noflt0:
a0l      = a0dl
a0r      = a0dr

; --------------------------------------------------------------------------------
;                             R E G I S T E R   1
; --------------------------------------------------------------------------------
         reg1:
         ; Initialisierung
i1total  = 0
a1m      = 0
a1l      = 0
a1r      = 0
         if i1i == 0 goto reg2

;--------------------------------------------------------------- i-rate action ---
	 ; Konstanten
i1ft     = ift1       ; f-Tab Nummer f. Sinusoszillator
i1ftvib  = ift5       ; f-Tab f. Vibrato
iNwid    = 1200       ; Baubreite [mm]
iNdst    = 1000       ; Hoerabstand [mm]

         ; Lokalisierung
iNx0     = ixoctav*(isze*iNwid/100.0)        ; x,y-Position der Schallquelle
iNy0     = 0                                 ; innerhalb des Instrumentes
iNx      = iNx0*icang-(idst*iNdst/100.0)*isazm-iNy0*isang ; x,y-Position der Schallquelle
iNy      = (idst*iNdst/100.0)*icazm+iNx0*isang+iNy0*icang ; relativ zum Zuhoerer
iNxl     = iNx+ihead/2                       ; x-Position relativ zum linken Ohr
iNxr     = iNx-ihead/2                       ; x-Position relativ zum rechten Ohr 
iNdstl2  = iNy^2+iNxl^2                      ; Abstand linkes Ohr hoch 2
iNdstr2  = iNy^2+iNxr^2                      ; Abstand rechtes Ohr hoch 2
iNdstl   = sqrt(iNdstl2)                     ; Abstand linkes Ohr [mm]
iNdstr   = sqrt(iNdstr2)                     ; Abstand rechtes Ohr [mm]
iNpha    = (iNdstr-iNdstl)/ivsnd             ; Phasendifferenz rechts - links [s]
iNilx    = (iNxl<=0?iihere:iiaway)           ; x-Richtungskoeffizient linkes Ohr
iNirx    = (iNxr>=0?iihere:iiaway)           ; x-Richtungskoeffizient rechtes Ohr
iNiy     = (iNy>0?iiface:iiback)             ; y-Richtungskoeffizient links u. rechts
iNhlx    = (iNxl<=0?ihhere:ihaway)           ; x-Grenzfrequenz linkes Ohr
iNhrx    = (iNxr>=0?ihhere:ihaway)           ; x-Grenzfrequenz rechtes Ohr
iNhy     = (iNy>0?ihface:ihback)             ; y-Grenzfrequenz links u. rechts
         ; Gesamtrichtungskoeffizienten links u. rechts
iNil     = (iNdstl==0?iNilx:sqrt((iNilx*iNxl^2+iNiy*iNy^2)/iNdstl^2)); 
iNir     = (iNdstr==0?iNirx:sqrt((iNirx*iNxr^2+iNiy*iNy^2)/iNdstr^2)); 
i1hl     = (iNdstl==0?iNhlx:sqrt((iNhlx*iNxl^2+iNhy*iNy^2)/iNdstl^2)); 
i1hr     = (iNdstr==0?iNhrx:sqrt((iNhrx*iNxr^2+iNhy*iNy^2)/iNdstr^2)); 
         ; amplitude modification by distance
iNdl     = (iNdstl>iNdst?iNdst/iNdstl:.5*(3-(iNdstl/iNdst)^2))
iNdr     = (iNdstr>iNdst?iNdst/iNdstr:.5*(3-(iNdstr/iNdst)^2))

         ; Daempfungskoeffizienten abhaengig von Tonhoehe
         ; Anschlagdauer, Anschlagstaerke
iNa      = 1+8*(icps/4000)^2               ; frequenzabhaengiger Ausgangswert
iNa      = iNa*(1+.2*exp(ivol/76.0-1))     ; abhaengig von Anschlagstaerke
iNa      = iNa*(.6+.6*exp(-idur))          ; abhaengig von Anschlagsdauer
iNb      = iNa                             ; Freischwingdaempfung
iNc      = 20*iNa                          ; Daempferdaempfung

         ; Anstiegs-/Freischwing-/Daempfungs-Amplituden und -Zeiten 
         ; abhaengig von Tonhoehe, Anschlagstaerke, Anschlagsdauer
i1start  = 1+((ioct-9)^2)/48               ; relative Anfangsamplitude
i1rise   = (12-ivol/19)/icps               ; Anstiegszeit
i1rise   = (i1rise<.001?.001:i1rise)       ; Minimum einhalten
i1rise   = (i1rise>(idur/2)?idur/2:i1rise) ; Maximum einhalten
iNfree   = idur - i1rise                   ; Freischwingzeit
i1rest   = i1start*exp(-iNb*iNfree)        ; relative Freischwing-Restamplitude
i1rest   = (i1rest==0?0.000001:i1rest)     ; darf nicht 0 sein wegen envlpx
i1dec    = -(log(iend)+iNb*iNfree)/iNc     ; Daempfungszeit
i1dec    = (i1dec < 0 ? .001 : i1dec)      ; wenn iend schon in Freischwing-Phase
i1total  = i1rise + iNfree + i1dec         ; Spieldauer

         ; Vibratoamplitude und Frequenz abhaengig von 
         ; Tonhoehe und Anschlagstaerke
i1vamp   = (ivibrato!=0?(ivol<ivgood?(idur<1?0:1):0):0)   ; Amplitude
i1vcps   = 4.5+0.1*ioct                                   ; Frequenz
i1vtime  = .333*idur                                      ; Zeitpunkt

         ; Frequenzabweichung abhaengig von Anschlag
i1dev    = 0.0006*((ivol/90)^2) ; relative Abweichung fuer Frequenzverlauf

         ; spacial signal modifications
i1left   = (idiramp!=0?.5*iNil:.5)            ; left amplitude modified by direction
i1right  = (idiramp!=0?.5*iNir:.5)            ; right amplitude modified by direction
i1left   = (idstamp!=0?iNdl*i1left:i1left)    ; left amplitude modified by distance
i1right  = (idstamp!=0?iNdr*i1right:i1right)  ; right amplitude modified by distance
i1dell   = (idirpha!=0?(iNpha>0?id0:id0-iNpha):id0) ; left phase modified by direction
i1delr   = (idirpha!=0?(iNpha>0?id0+iNpha:id0):id0) ; right phase modified by direction

;--------------------------------------------------------------- k-rate action ---
         ; Einhuellende
kenv     envlpx i1start, i1rise, i1total, i1dec, ift8, i1rest/i1start, iend/i1rest
kenv     = (ktime>i1total?0:kenv)

         ; Vibratoamplitude
kvosc    poscil i1vamp, i1vcps, i1ftvib, 0.25
kvenv    linseg 0, i1vtime, 0, i1total-i1vtime, 1
kvenv    = (ktime>i1total?0:kvenv)
kvib     = 1+kvenv*kvosc

         ; resultierender Amplitudenverlauf
k1amp    = imix*iamp*kvib*kenv
	
         ; Frequenzverlauf
k1fenv   linseg 1+i1dev, i1total, 1

;--------------------------------------------------------------- a-rate action ---
         ; Signalerzeugung
a1m      poscil k1amp, k1fenv*icps, i1ft, ipha
a1dl     delay i1left*a1m, i1dell
a1dr     delay i1right*a1m, i1delr
	 if idirflt == 0 goto noflt1
a1l      tone a1dl, i1hl
a1r      tone a1dr, i1hr
         goto reg2
         noflt1:
a1l      = a1dl
a1r      = a1dr

; --------------------------------------------------------------------------------
;                             R E G I S T E R   2
; --------------------------------------------------------------------------------
         reg2:
         ; Vorbelegung (falls Rest wegen i2i == 0 entfaellt)
i2total  = 0
a2m      = 0
a2l      = 0
a2r      = 0
         if i2i == 0 goto reg3 ; ---------------> weiter mit Register 3

;--------------------------------------------------------------- i-rate action ---
         ; Konstanten
i2ft     = ift3       ; f-Tab Nummer f. FM-Oszillator
i2ftvib  = ift5       ; f-Tab f. Vibrato
i2car    = 1          ; relative FM-Traegerfrequenz
i2mod    = 2.02       ; relative FM-Modulationsfrequenz
i2ind    =  .02       ; FM-Modulationsindex
iNwid    = 1200       ; Baubreite [mm]
iNdst    = 1000       ; Hoerabstand [mm]

         ; Lokalisierung
iNx0     = ixoctav*(isze*iNwid/100.0)        ; x,y-Position der Schallquelle
iNy0     = 0;                                ; innerhalb des Instrumentes
iNx      = iNx0*icang-(idst*iNdst/100.0)*isazm-iNy0*isang ; x,y-Position der Schallquelle
iNy      = (idst*iNdst/100.0)*icazm+iNx0*isang+iNy0*icang ; relativ zum Zuhoerer
iNxl     = iNx+ihead/2                       ; x-Position relativ zum linken Ohr
iNxr     = iNx-ihead/2                       ; x-Position relativ zum rechten Ohr 
iNdstl2  = iNy^2+iNxl^2                      ; Abstand linkes Ohr hoch 2
iNdstr2  = iNy^2+iNxr^2                      ; Abstand rechtes Ohr hoch 2
iNdstl   = sqrt(iNdstl2)                     ; Abstand linkes Ohr [mm]
iNdstr   = sqrt(iNdstr2)                     ; Abstand rechtes Ohr [mm]
iNpha    = (iNdstr-iNdstl)/ivsnd             ; Phasendifferenz rechts - links [s]
iNilx    = (iNxl<=0?iihere:iiaway)           ; x-Richtungskoeffizient linkes Ohr
iNirx    = (iNxr>=0?iihere:iiaway)           ; x-Richtungskoeffizient rechtes Ohr
iNiy     = (iNy>0?iiface:iiback)             ; y-Richtungskoeffizient links u. rechts
iNhlx    = (iNxl<=0?ihhere:ihaway)           ; x-Grenzfrequenz linkes Ohr
iNhrx    = (iNxr>=0?ihhere:ihaway)           ; x-Grenzfrequenz rechtes Ohr
iNhy     = (iNy>0?ihface:ihback)             ; y-Grenzfrequenz links u. rechts
         ; Gesamtrichtungskoeffizienten links u. rechts
iNil     = (iNdstl==0?iNilx:sqrt((iNilx*iNxl^2+iNiy*iNy^2)/iNdstl^2)); 
iNir     = (iNdstr==0?iNirx:sqrt((iNirx*iNxr^2+iNiy*iNy^2)/iNdstr^2)); 
i2hl     = (iNdstl==0?iNhlx:sqrt((iNhlx*iNxl^2+iNhy*iNy^2)/iNdstl^2)); 
i2hr     = (iNdstr==0?iNhrx:sqrt((iNhrx*iNxr^2+iNhy*iNy^2)/iNdstr^2)); 
         ; amplitude modification by distance
iNdl     = (iNdstl>iNdst?iNdst/iNdstl:.5*(3-(iNdstl/iNdst)^2))
iNdr     = (iNdstr>iNdst?iNdst/iNdstr:.5*(3-(iNdstr/iNdst)^2))

         ; Daempfungskoeffizienten abhaengig von Tonhoehe
         ; Anschlagdauer, Anschlagstaerke
iNa      = 1+8*(icps/4000)^2               ; frequenzabhaengiger Ausgangswert
iNa      = iNa*(1+.2*exp(ivol/76.0-1))     ; abhaengig von Anschlagstaerke
iNa      = iNa*(.6+.6*exp(-idur))          ; abhaengig von Anschlagsdauer
iNb      = iNa                             ; Freischwingdaempfung
iNc      = 20*iNa                          ; Daempferdaempfung

         ; Anstiegs-/Freischwing-/Daempfungs-Amplituden und -Zeiten 
         ; abhaengig von Tonhoehe, Anschlagstaerke, Anschlagsdauer
i2start  = 1+((ioct-9)^2)/48               ; relative Anfangsamplitude
i2rise   = (12-ivol/19)/icps               ; Anstiegszeit
i2rise   = (i2rise<.001?.001:i2rise)       ; Minimum einhalten
i2rise   = (i2rise>(idur/2)?idur/2:i2rise) ; Maximum einhalten
iNfree   = idur - i2rise                   ; Freischwingzeit
i2rest   = i2start*exp(-iNb*iNfree)        ; relative Freischwing-Restamplitude
i2rest   = (i2rest==0?0.000001:i2rest)     ; darf nicht 0 sein wegen envlpx
i2dec    = -(log(iend)+iNb*iNfree)/iNc     ; Daempfungszeit
i2dec    = (i2dec < 0 ? .001 : i2dec)      ; wenn iend schon in Freischwing-Phase
i2total  = i2rise + iNfree + i2dec         ; Spieldauer

         ; Vibratoamplitude und Frequenz abhaengig von 
         ; Tonhoehe und Anschlagstaerke
i2vamp   = (ivibrato!=0?(ivol<ivgood?(idur<1?0:1):0):0)   ; Amplitude
i2vcps   = 4.5+0.1*ioct                                   ; Frequenz
i2vtime  = .333*idur                                      ; Zeitpunkt

         ; Frequenzabweichung abhaengig von Anschlag
i2dev    = (2^(1/1200)-1)*((ivol/90)^2)    ; relative Abweichung fuer Frequenzverlauf

         ; Modulationsindex abhaengig von Oktave
;i2ind    = (ioct<10?(10-ioct)/8:0)         ; FM-Modulationsindex

         ; spacial signal modifications
i2left   = (idiramp!=0?.5*iNil:.5)            ; left amplitude modified by direction
i2right  = (idiramp!=0?.5*iNir:.5)            ; right amplitude modified by direction
i2left   = (idstamp!=0?iNdl*i2left:i2left)    ; left amplitude modified by distance
i2right  = (idstamp!=0?iNdr*i2right:i2right)  ; right amplitude modified by distance
i2dell   = (idirpha!=0?(iNpha>0?id0:id0-iNpha):id0) ; left phase modified by direction
i2delr   = (idirpha!=0?(iNpha>0?id0+iNpha:id0):id0) ; right phase modified by direction

;--------------------------------------------------------------- k-rate action ---
         ; Einhuellende
kenv     envlpx i2start, i2rise, i2total, i2dec, ift8, i2rest/i2start, iend/i2rest
kenv     = (ktime>i2total?0:kenv)

         ; Vibratoamplitude
kvosc    poscil i2vamp, i2vcps, i2ftvib
kvenv    linseg 0, i2vtime, 0, i2total-i2vtime, 1
kvenv    = (ktime>i2total?0:kvenv)
kvib     = 1+kvenv*kvosc

         ; resultierender Amplitudenverlauf
k2amp    = imix*iamp*kvib*kenv

         ; Frequenzverlauf
k2fenv   linseg 1+i2dev, i2total, 1

         ; Modulationsindexverlauf
;k2ind    linseg 0, i2total, i2ind
;k2ind    = (ktime>i2total?i2ind/2:k2ind)

;--------------------------------------------------------------- a-rate action ---
         ; Signalerzeugung
a2m      foscili k2amp, k2fenv*icps, i2car, i2mod, i2ind, i2ft, ipha
a2dl     delay i2left*a2m, i2dell
a2dr     delay i2right*a2m, i2delr
	 if idirflt == 0 goto noflt2
a2l      tone a2dl, i2hl
a2r      tone a2dr, i2hr
         goto reg3
         noflt2:
a2l      = a2dl
a2r      = a2dr

; --------------------------------------------------------------------------------
;                             R E G I S T E R   3
; --------------------------------------------------------------------------------
         reg3:
         ; Vorbelegung (falls Rest wegen i3i == 0 entfaellt)
i3total  = 0
a3m      = 0
a3l      = 0
a3r      = 0
         if i3i == 0 goto reg4 ; -----------------> weiter mit Register 4

;--------------------------------------------------------------- i-rate action ---
         ; Konstanten
i3ft     = ift4       ; f-Tab Nummer f. Buzzer
i3ftvib  = ift5       ; f-Tab f. Vibrato
iNwid    = 1200       ; Baubreite [mm]
iNdst    = 1000       ; Hoerabstand [mm]

         ; Lokalisierung
iNx0     = ixoctav*(isze*iNwid/100.0)        ; x,y-Position der Schallquelle
iNy0     = 0;                                ; innerhalb des Instrumentes
iNx      = iNx0*icang-(idst*iNdst/100.0)*isazm-iNy0*isang ; x,y-Position der Schallquelle
iNy      = (idst*iNdst/100.0)*icazm+iNx0*isang+iNy0*icang ; relativ zum Zuhoerer
iNxl     = iNx+ihead/2                       ; x-Position relativ zum linken Ohr
iNxr     = iNx-ihead/2                       ; x-Position relativ zum rechten Ohr 
iNdstl2  = iNy^2+iNxl^2                      ; Abstand linkes Ohr hoch 2
iNdstr2  = iNy^2+iNxr^2                      ; Abstand rechtes Ohr hoch 2
iNdstl   = sqrt(iNdstl2)                     ; Abstand linkes Ohr [mm]
iNdstr   = sqrt(iNdstr2)                     ; Abstand rechtes Ohr [mm]
iNpha    = (iNdstr-iNdstl)/ivsnd             ; Phasendifferenz rechts - links [s]
iNilx    = (iNxl<=0?iihere:iiaway)           ; x-Richtungskoeffizient linkes Ohr
iNirx    = (iNxr>=0?iihere:iiaway)           ; x-Richtungskoeffizient rechtes Ohr
iNiy     = (iNy>0?iiface:iiback)             ; y-Richtungskoeffizient links u. rechts
iNhlx    = (iNxl<=0?ihhere:ihaway)           ; x-Grenzfrequenz linkes Ohr
iNhrx    = (iNxr>=0?ihhere:ihaway)           ; x-Grenzfrequenz rechtes Ohr
iNhy     = (iNy>0?ihface:ihback)             ; y-Grenzfrequenz links u. rechts
         ; Gesamtrichtungskoeffizienten links u. rechts
iNil     = (iNdstl==0?iNilx:sqrt((iNilx*iNxl^2+iNiy*iNy^2)/iNdstl^2)); 
iNir     = (iNdstr==0?iNirx:sqrt((iNirx*iNxr^2+iNiy*iNy^2)/iNdstr^2)); 
i3hl     = (iNdstl==0?iNhlx:sqrt((iNhlx*iNxl^2+iNhy*iNy^2)/iNdstl^2)); 
i3hr     = (iNdstr==0?iNhrx:sqrt((iNhrx*iNxr^2+iNhy*iNy^2)/iNdstr^2)); 
         ; amplitude modification by distance
iNdl     = (iNdstl>iNdst?iNdst/iNdstl:.5*(3-(iNdstl/iNdst)^2))
iNdr     = (iNdstr>iNdst?iNdst/iNdstr:.5*(3-(iNdstr/iNdst)^2))

         ; Daempfungskoeffizienten abhaengig von Tonhoehe
         ; Anschlagdauer, Anschlagstaerke
iNa      = 1+8*(icps/4000)^2               ; frequenzabhaengiger Ausgangswert
iNa      = iNa*(1+.2*exp(ivol/76.0-1))     ; abhaengig von Anschlagstaerke
iNa      = iNa*(.6+.6*exp(-idur))          ; abhaengig von Anschlagsdauer
iNb      = iNa                             ; Freischwingdaempfung
iNc      = 20*iNa                          ; Daempferdaempfung

         ; Anstiegs-/Freischwing-/Daempfungs-Amplituden und -Zeiten 
         ; abhaengig von Tonhoehe, Anschlagstaerke, Anschlagsdauer
i3start  = 1+((ioct-9)^2)/48               ; relative Anfangsamplitude
i3rise   = (12-ivol/19)/icps               ; Anstiegszeit
i3rise   = (i3rise<.001?.001:i3rise)       ; Minimum einhalten
i3rise   = (i3rise>(idur/2)?idur/2:i3rise) ; Maximum einhalten
iNfree   = idur - i3rise                   ; Freischwingzeit
i3rest   = i3start*exp(-iNb*iNfree)        ; relative Freischwing-Restamplitude
i3rest   = (i3rest==0?0.000001:i3rest)     ; darf nicht 0 sein wegen envlpx
i3dec    = -(log(iend)+iNb*iNfree)/iNc     ; Daempfungszeit
i3dec    = (i3dec < 0 ? .001 : i3dec)      ; wenn iend schon in Freischwing-Phase
i3total  = i3rise + iNfree + i3dec         ; Spieldauer

         ; Vibratoamplitude und Frequenz abhaengig von 
         ; Tonhoehe und Anschlagstaerke
i3vamp   = (ivibrato!=0?(ivol<ivgood?(idur<1?0:1):0):0)   ; Amplitude
i3vcps   = 4.5+0.1*ioct                                   ; Frequenz
i3vtime  = .333*idur                                      ; Zeitpunkt

         ; Frequenzabweichung abhaengig von Anschlag
i3dev    = 0.0006*((ivol/90)^2) ; relative Abweichung fuer Frequenzverlauf

         ; spacial signal modifications
i3left   = (idiramp!=0?.5*iNil:.5)            ; left amplitude modified by direction
i3right  = (idiramp!=0?.5*iNir:.5)            ; right amplitude modified by direction
i3left   = (idstamp!=0?iNdl*i3left:i3left)    ; left amplitude modified by distance
i3right  = (idstamp!=0?iNdr*i3right:i3right)  ; right amplitude modified by distance
i3dell   = (idirpha!=0?(iNpha>0?id0:id0-iNpha):id0) ; left phase modified by direction
i3delr   = (idirpha!=0?(iNpha>0?id0+iNpha:id0):id0) ; right phase modified by direction

         ; Summerparameter
i3nharm  = int(sr/icps/2.0)                 ; Anzahl Harmonien
i3nharm  = (i3nharm<1?1:i3nharm)            ; mindestens eine
i3nharm  = (i3nharm>64?64:i3nharm)          ; begrenzen auf 64
i3kr     = 0.5*exp(-0.697*icps/160)+0.4     ; Amplitudenabfall
i3side   = 0.5*exp(-0.697*icps/60)          ; relative Amplitude Seitenband

;--------------------------------------------------------------- k-rate action ---
         ; Einhuellende
kenv     envlpx i3start, i3rise, i3total, i3dec, ift8, i3rest/i3start, iend/i3rest
kenv     = (ktime>i3total?0:kenv)

         ; Vibratoamplitude
kvosc    poscil i3vamp, i3vcps, i3ftvib, 0.25
kvenv    linseg 0, i3vtime, 0, i3total-i3vtime, 1
kvenv    = (ktime>i3total?0:kvenv)
kvib     = 1+kvenv*kvosc

         ; resultierender Amplitudenverlauf
k3amp    = imix*iamp*kvib*kenv

         ; Frequenzverlauf
k3fenv   linseg 1+i3dev, i3total, 1

         ; Seitenbandverlauf
k3side   linseg 0, i3total, i3side
k3main   = 1.0 - k3side

;--------------------------------------------------------------- a-rate action ---
         ; Signalerzeugung
a3main   gbuzz k3amp, k3fenv*icps, i3nharm, 1, i3kr, i3ft, ipha
a3side   gbuzz k3amp, k3fenv*icps/2.0, 2.0*i3nharm, 2, i3kr, i3ft, ipha
a3m      = k3main*a3main + k3side*a3side
a3dl     delay i3left*a3m, i3dell
a3dr     delay i3right*a3m, i3delr
	 if idirflt == 0 goto noflt3
a3l      tone a3dl, i3hl
a3r      tone a3dr, i3hr
         goto reg4
         noflt3:
a3l      = a3dl
a3r      = a3dr         

; --------------------------------------------------------------------------------
;                             R E G I S T E R   4
; --------------------------------------------------------------------------------
         reg4:
; --------------------------------------------------------------------------------
;                           G E S A M T S I G N A L
; --------------------------------------------------------------------------------

         ; Gesamtspieldauer berechnen
itotal   = i0total
itotal   = (i1total>itotal?i1total:itotal)
itotal   = (i2total>itotal?i2total:itotal)
itotal   = (i3total>itotal?i3total:itotal)

         ; Originalparameter ueberschreiben
p3       = itotal;

         ; Registeramplituden auf 1 normieren
iitotal  = i0i + i1i + i2i + i3i
i0i      = i0i/iitotal
i1i      = i1i/iitotal
i2i      = i2i/iitotal
i3i      = i3i/iitotal

         ; compose signals main, left, right
amain    delay i0i*a0m + i1i*a1m + i2i*a2m + i3i*a3m, id0
aleft    = i0i*a0l + i1i*a1l + i2i*a2l + i3i*a3l 
aright   = i0i*a0r + i1i*a1r + i2i*a2r + i3i*a3r

         ; signal output
outs     aleft, aright

endin ; --------------------------------------------------------------------------
