#!/usr/bin/tclsh
########################################################################
# Copyright (c) 2004 by Hartmut Schwarz, matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
########################################################################

# makes the HTML track list

########################################################################
# configuration section                                                #
########################################################################
set country "EN";
set pattern "tracklist_$country";

########################################################################
# function definition section                                          #
########################################################################

# load common functions
source various.tcl
source wavinfo.tcl

# print a minimum of help information
proc printhelp { name } {
 puts "$name : create HTML song index";
 puts "usage : $argv0 \[-ST|-MA|-IR\] score1 score2 score3 ... <templatefile";
 puts "        -IR = left/right (stereo) + main signal room response";
 puts "        -MA = main signal (mono) without room response";
 puts "        -ST = left/right (stereo) without room response";
}

# English/German text translation
proc translate { text } {
 global country trackinfo;
 if { $country == "DE" } {
  regsub -all {M\+(<.*>)T:} $text {Musik und \1Text:} text;
  regsub -all {T:} $text {Text:} text;
  regsub -all {M:} $text {Musik:} text;
  regsub -all {traditional from} $text {volkstmlich aus} text;
  regsub -all {France} $text {Frankreich} text;
  regsub -all {Russia} $text {Russland} text;
  regsub -all {by} $text {von} text;
  regsub -all {unknown} $text {Herkunft unbekannt} text;
 } else {
  regsub -all {M\+(<.*>)T:} $text {Music and \1lyrics:} text;
  regsub -all {T:} $text {Lyrics:} text;
  regsub -all {M:} $text {Music:} text;
  regsub -all {unknown} $text {unknown origin} text;
 }
 return $text;
}

# get track length (mm:ss)
proc get_length { track } {
 global dirwav;
 return [evaltracktime $dirwav/$track.wav];
}

# get track title
proc get_title { track } {
 global country trackinfo;
  if { [catch { exec $trackinfo $track title$country } result] } {
  set result [exec $trackinfo $track TITLE];
 }
 return "<b><a href=\"mp3/$track.mp3\">$result</a></b>";
}

# get additional track info
proc get_info { track } {
 global trackinfo;
 # melody, lyrics, text holen
  if { [catch { exec $trackinfo $track melody } melody] } {
  set melody "unknown";
 }
  if { [catch { exec $trackinfo $track lyrics } lyrics] } {
  set lyrics "unknown";
 }
  if { [catch { exec $trackinfo $track text } text] } {
  set text "unknown";
 }
 # Info zusammenstellen
 if { $text == "unknown" } {
  if { $melody == "unknown" } {
   set info "";
  } elseif { ![regexp {traditional} $melody] } {
   set info "\nby $melody";
  } else {
   set info "\n$melody";
  }
 } else {
  if { $melody == $lyrics } {
   set info "\nM+<a href=\"$track.htm\">T:</a> $melody ";
  } else {
   set info "\nM: $melody<br />\n<a href=\"$track.htm\">T:</a> $lyrics";
  }
 }
 if { $info != "" } {
  set info "<br />[ translate $info]";
 }
 return $info;
}

# get additional track IR convolution info
proc get_IR_info { track } {
 global country;
 source "../info/default.convinfo.tcl";
 if { [file isfile "../info/convinfo.$track.tcl"] } {
  source "../info/convinfo.$track.tcl";
 }
 set amorig [expr $amcomm*$amorig];
 set amconv [expr $amcomm*$amconv];
 if { $country == "DE" } {
  if { $impnam == "none" } {
   set irinfo "ohne Raumantwort"
  } else {
   set irinfo "$amorig direkt + $amconv IR <a href=\"../contributions/$impnam.wav\">$impnam.wav</a>"
   if { $intime != 0 } {
    set irinfo "$irinfo, Einblendzeit $intime%";
   }
   if { $outime != 0 } {
    set irinfo "$irinfo, Ausblendzeit $outime%";
   }
  }
 } else {
  if { $impnam == "none" } {
   set irinfo "without room response"
  } else {
   set irinfo "$amorig direct + $amconv IR <a href=\"../contributions/$impnam.wav\">$impnam.wav</a>"
   if { $intime != 0 } {
    set irinfo "$irinfo, fade in time $intime%";
   }
   if { $outime != 0 } {
    set irinfo "$irinfo, fade out time $outime%";
   }
  }
 }
 return $irinfo;
}

# build comment from all kinds of track info
proc get_comment { track } {
 global country trackinfo mixmode;
 if { [catch { exec $trackinfo $track comment$country } comment] } {
  if { $mixmode == "-IR" } {
   set comment "<br />\n([get_IR_info $track])";
  } else {
   set comment ""
  }
 } else {
  if { $mixmode == "-IR" } {
   set comment "<br />\n$comment<br />\n([get_IR_info $track])";
  } else {
   set comment "<br />\n$comment"
  }
 }
 return $comment;
}

# get a complete entry for the track index
proc get_entry { track } {
# holt einen Liedeintrag
 set text "\n<li>\[[get_length $track]\] [get_title $track][get_info $track][get_comment $track]</li>";
 return [replace_entities $text];
}

# get common sound mixer info
proc get_mix_info {} {
 global country mixmode stereoflag monoflag;
 set DEir "Mix: linker und rechter Kanal (stereo) mit Raumantwort des Hauptsignals";
 set DEma "Mix: Hauptsignal (mono) ohne Raumantwort";
 set DEst "Mix: linker und rechter Kanal (stereo) ohne Raumantwort";
 set ENir "mix: left and right channel (stereo) plus main signal room response";
 set ENma "mix: main signal (mono) without room response";
 set ENst "mix: left and right channel (stereo) without room response";
 if { $mixmode == "-IR" } {
  return [set ${country}ir];
 } elseif { $mixmode == "-MA" } {
  return [set ${country}ma];
 } else {
  return [set ${country}st];
 }
 return $text
}

# get the complete track list
proc get_track_list { argv lang } {
# HTML Listenfragment erstellen
 global country trackinfo dirwav;
 set country $lang;
 set text "<ol>\n";
 set totaltime 0;
 foreach track $argv {
  set text "$text[get_entry $track]\n";
  if { $totaltime != "??:??" && [catch { evalnumsamples $dirwav/$track.wav } result] == 0 } {
   incr totaltime [samples2seconds $result];
  } else {
   set totaltime "??:??";
  }
 }
 if { $totaltime != "??:??" } {
  set totaltime [seconds2tracktime $totaltime];
 }
 if { $lang == "DE" } {
  set text "$text</ol>\n<p>\[$totaltime\] Gesamtspielzeit (Min:Sek)\n<br />[get_mix_info]</p>\n";
 } else {
  set text "$text</ol>\n<p>\[$totaltime\] total play time (min:sec)\n<br />[get_mix_info]</p>\n";
 }
 return $text;
}

########################################################################
# program execution section                                            #
########################################################################

# evaluate mixer mode
set mixmode [eval_mix_mode];
if { $mixmode != "-MA" && $mixmode != "-ST" && $mixmode != "-IR" } {
 printhelp $argv0;
 exit 1;
}
set argc [llength $argv];

# select WAV directory according to mixer mode
if { $mixmode == "-IR" } {
 set dirwav  "[file dirname $argv0]/../../music3";
} elseif { $mixmode == "-MA" } {
 set dirwav  "[file dirname $argv0]/../../music1";
} else {
 set dirwav  "[file dirname $argv0]/../../music2";
}

# turn WAV directory name into a full pathname without ..
set dirwav [file normalize $dirwav];

# check number of arguments
if { $argc < 1 } {
 printhelp $argv0;
 exit 2;
}

# check for each track having title info at least
set trackinfo "[file dirname $argv0]/trackinfo.tcl" ;
foreach track $argv {
  if { [catch { exec $trackinfo $track TITLE } result] } {
  puts stderr "*** no TITLE for track \"$track\" ***";
  exit 3;
 }
};

# transfer HTML-template from stdin to stdout
# and replace "<? tracklist ?>" by track list.

while { [gets stdin line] >= 0 } {
 if { [regsub {<\? tracklist \?>} $line {[get_track_list $argv EN]} line] } {
  set line [subst $line];
 }
 if { [regsub {<\? Liedindex \?>} $line {[get_track_list $argv DE]} line] } {
  set line [subst $line];
 }
 puts $line
};
