#!/usr/bin/tclsh
########################################################################
# Copyright (c) 2004 by Hartmut Schwarz, matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
########################################################################

# makes HTML song lyrics document

########################################################################
# configuration section                                                #
########################################################################
set listdocDE "Deutsch.htm"
set listdocEN "English.htm"

########################################################################
# function definition section                                          #
########################################################################

# load common functions
source various.tcl

# get HTML document title
proc get_doc_title {} {
 global track trackinfo;
 if { [catch { exec $trackinfo $track titleEN } result] } {
  if { [catch { exec $trackinfo $track titleDE } result] } {
   set result [exec $trackinfo $track TITLE];
  }
 }
 return [replace_entities $result];
}

# get song title
proc get_song_title {} {
 global track trackinfo;
 set result [exec $trackinfo $track TITLE];
 return [replace_entities $result];
}

# get info about author and composer
proc get_info {} {
 global track trackinfo;
 if { [catch { exec $trackinfo $track melody } melody] } {
  set melody "unknown origin";
 }
 if { [catch { exec $trackinfo $track lyrics } lyrics] } {
  set lyrics "unknown origin";
 }
 if { $melody == $lyrics } {
  set info "M+T: $melody";
 } else {
  set info "M: $melody,<br />\nT: $lyrics";
 }
 return [replace_entities $info];
}

# get HTML song info and lyrics
proc get_html_text {} {
 global track trackinfo;
 set text "<p>[exec $trackinfo $track text]</p>";
 regsub -all {([^\n])(\n)([^\n])} $text {\1<br />\2\3} text;
 regsub -all {\n\n} $text {</p>&<p>} text;
 return [replace_entities $text];
}

########################################################################
# program execution section                                            #
########################################################################

# check input
if { $argc != 1 } {
 puts "$argv0 : create HTML lyrics document";
 puts "usage : $argv0 scorename";
 exit 1;
} else {
 set track [lindex $argv 0];
}

# find title and lyrics
set trackinfo "[file dirname $argv0]/trackinfo.tcl" ;
if { [catch { exec $trackinfo $track TITLE } result] } {
 puts stderr "*** no TITLE for track \"$track\" ***";
 exit 2;
} elseif { [catch { exec $trackinfo $track text } result] } {
 puts stderr "*** no text for track \"$track\" ***";
 exit 3;
};

# HTML-Ausgabe
puts "<html>\n<head>\n<title>[get_doc_title]</title>\n</head>\n<body>\n";
puts "<p><a href=\"$listdocDE\">zur&uuml;ck</a> | <a href=\"$listdocEN\">back</a></p>\n";
puts "<h1>[get_song_title]</h1>\n";
puts "<p><i>[get_info]</i></p>\n";
puts "[get_html_text]\n";
puts "<p><a href=\"$listdocDE\">zur&uuml;ck</a> | <a href=\"$listdocEN\">back</a></p>\n";
puts "</body>\n</html>\n";
